In-line run of subs_in_out so that it gets the environment more easily.
[libdcp.git] / src / cpl.h
index 5d087c039da724e76d3bc9a6d606487b276f06fb..824faaa132217756054a8d5fad66aefe76b985c1 100644 (file)
--- a/src/cpl.h
+++ b/src/cpl.h
@@ -33,7 +33,7 @@
 
 
 /** @file  src/cpl.h
- *  @brief CPL class.
+ *  @brief CPL class
  */
 
 
 
 #include "asset.h"
 #include "certificate.h"
+#include "content_kind.h"
 #include "key.h"
 #include "language_tag.h"
-#include "types.h"
+#include "rating.h"
+#include "verify.h"
 #include <boost/filesystem.hpp>
 #include <boost/function.hpp>
 #include <boost/optional.hpp>
 #include <vector>
 
 
-struct verify_various_invalid_languages;
+struct verify_invalid_language3;
 
 
 namespace dcp {
 
 
-class ReelMXF;
-class Reel;
-class MXFMetadata;
 class CertificateChain;
 class DecryptedKDM;
+class MXFMetadata;
+class Reel;
+class ReelFileAsset;
+class SoundAsset;
 
 
 /** @class CPL
- *  @brief A Composition Playlist.
+ *  @brief A Composition Playlist
+ *
+ *  A CPL contains some metadata and a list of Reel objects, each of which may contain picture, sound and other assets
+ *  such as subtitles and closed captions.
+ *
+ *  After creating a CPL you can add Reel objects with add(), and write an XML file describing the CPL with
+ *  write_xml().
  */
 class CPL : public Asset
 {
 public:
-       CPL (std::string annotation_text, ContentKind content_kind);
-       explicit CPL (boost::filesystem::path file);
+       CPL (std::string annotation_text, ContentKind content_kind, Standard standard);
+
+       /** Construct a CPL object from a XML file.
+        *  If notes is not null, non-fatal errors will be added.
+        *  Exceptions will be thrown on non-recoverable errors.
+        */
+       explicit CPL(boost::filesystem::path file, std::vector<dcp::VerificationNote>* notes = nullptr);
 
        bool equals (
                std::shared_ptr<const Asset> other,
-               EqualityOptions options,
+               EqualityOptions const& options,
                NoteHandler note
-               ) const;
+               ) const override;
 
+       /** Add a reel to this CPL
+        *  @param reel Reel to add
+        */
        void add (std::shared_ptr<Reel> reel);
+
+       void set (std::vector<std::shared_ptr<Reel>> reels);
+
+       /** Add a KDM to this CPL.  If the KDM is for any of this CPLs assets it will be used
+        *  to decrypt those assets.
+        *  @param kdm KDM.
+        */
        void add (DecryptedKDM const &);
 
        /** @return the reels in this CPL */
@@ -89,27 +113,45 @@ public:
                return _reels;
        }
 
-       /** @return the ReelMXFs in this CPL in all reels */
-       std::vector<std::shared_ptr<const ReelMXF>> reel_mxfs () const;
-       std::vector<std::shared_ptr<ReelMXF>> reel_mxfs ();
+       /** @return the ReelFileAssets in this CPL in all reels */
+       std::vector<std::shared_ptr<const ReelFileAsset>> reel_file_assets () const;
+       std::vector<std::shared_ptr<ReelFileAsset>> reel_file_assets ();
 
+       /** @return true if we have any encrypted content */
        bool any_encrypted () const;
+
+       /** @return true if we have all our encryptable content is encrypted */
        bool all_encrypted () const;
 
+       /** Write a CompositionPlaylist XML file
+        *
+        *  @param file Filename to write
+        *  @param signer Signer to sign the CPL, or 0 to add no signature
+        *  @param include_mca_subdescriptors true to add a MCASubDescriptors tag to metadata,
+        *  false to omit it.
+        */
        void write_xml (
                boost::filesystem::path file,
-               Standard standard,
-               std::shared_ptr<const CertificateChain>
+               std::shared_ptr<const CertificateChain>,
+               bool include_mca_subdescriptors = true
                ) const;
 
        void resolve_refs (std::vector<std::shared_ptr<Asset>>);
 
        int64_t duration () const;
 
+       std::string issuer () const {
+               return _issuer;
+       }
+
        void set_issuer (std::string issuer) {
                _issuer = issuer;
        }
 
+       std::string creator () const {
+               return _creator;
+       }
+
        void set_creator (std::string creator) {
                _creator = creator;
        }
@@ -244,11 +286,11 @@ public:
                _luminance = l;
        }
 
-       boost::optional<std::string> main_sound_configuration () const {
+       boost::optional<dcp::MainSoundConfiguration> main_sound_configuration () const {
                return _main_sound_configuration;
        }
 
-       void set_main_sound_configuration (std::string c) {
+       void set_main_sound_configuration(dcp::MainSoundConfiguration c) {
                _main_sound_configuration = c;
        }
 
@@ -272,9 +314,7 @@ public:
                return _main_picture_active_area;
        }
 
-       void set_main_picture_active_area (dcp::Size s) {
-               _main_picture_active_area = s;
-       }
+       void set_main_picture_active_area(dcp::Size area);
 
        std::vector<std::string> additional_subtitle_languages () const {
                return _additional_subtitle_languages;
@@ -282,21 +322,45 @@ public:
 
        void set_additional_subtitle_languages (std::vector<dcp::LanguageTag> const& lang);
 
-       boost::optional<Standard> standard () const {
+       void set_sign_language_video_language (dcp::LanguageTag lang) {
+               _sign_language_video_language = lang.to_string();
+       }
+
+       boost::optional<std::string> sign_language_video_language () const {
+               return _sign_language_video_language;
+       }
+
+       void set_dolby_edr_image_transfer_function(std::string function) {
+               _dolby_edr_image_transfer_function = function;
+       }
+
+       boost::optional<std::string> dolby_edr_image_transfer_function() const {
+               return _dolby_edr_image_transfer_function;
+       }
+
+       Standard standard () const {
                return _standard;
        }
 
+       /** @return true iff this CPL was read from a file and it contained
+        *  a CompositionMetadataAsset node.
+        */
+       bool read_composition_metadata() const {
+               return _read_composition_metadata;
+       }
+
        static std::string static_pkl_type (Standard standard);
 
 protected:
        /** @return type string for PKLs for this asset */
-       std::string pkl_type (Standard standard) const;
+       std::string pkl_type (Standard standard) const override;
 
 private:
-       friend struct ::verify_various_invalid_languages;
+       friend struct ::verify_invalid_language3;
 
-       void maybe_write_composition_metadata_asset (xmlpp::Element* node) const;
+       void maybe_write_composition_metadata_asset(xmlpp::Element* node, bool include_mca_subdescriptors) const;
        void read_composition_metadata_asset (cxml::ConstNodePtr node);
+       void write_mca_subdescriptors(xmlpp::Element* parent, std::shared_ptr<const SoundAsset> asset) const;
 
        std::string _issuer;
        std::string _creator;
@@ -306,6 +370,10 @@ private:
        ContentKind _content_kind;                  ///< &lt;ContentKind&gt;
        std::vector<ContentVersion> _content_versions;
        std::vector<Rating> _ratings;
+       /** ID for CompositionMetadataAsset tag; either a random one, ready for writing a new tag,
+        *  or the one read in from the existing CPL.
+        */
+       std::string _cpl_metadata_id = make_uuid();
        /** Human-readable name of the composition, without any metadata (i.e. no -FTR-EN-XX- etc.) */
        boost::optional<std::string> _full_content_title_text;
        boost::optional<std::string> _full_content_title_text_language;
@@ -320,17 +388,20 @@ private:
        boost::optional<std::string> _distributor;
        boost::optional<std::string> _facility;
        boost::optional<Luminance> _luminance;
-       boost::optional<std::string> _main_sound_configuration;
+       boost::optional<MainSoundConfiguration> _main_sound_configuration;
        boost::optional<int> _main_sound_sample_rate;
        boost::optional<dcp::Size> _main_picture_stored_area;
        boost::optional<dcp::Size> _main_picture_active_area;
        /* See note for _release_territory above */
        std::vector<std::string> _additional_subtitle_languages;
+       boost::optional<std::string> _sign_language_video_language;
+       boost::optional<std::string> _dolby_edr_image_transfer_function;
+       bool _read_composition_metadata = false;
 
        std::vector<std::shared_ptr<Reel>> _reels;
 
        /** Standard of CPL that was read in */
-       boost::optional<Standard> _standard;
+       Standard _standard;
 };